
#include <stdint.h>
#include "c_types.h"
#include "esp8266_auxrom.h"
#include "esp8266_rom.h"
#include "eagle_soc.h"
#include "ets_sys.h"
#include "gpio.h"
#include "nosdk8266.h"
#include "ch.h"

#define call_delay_us( time ) { asm volatile( "mov.n a2, %0\n_call0 delay4clk" : : "r"(time*13) : "a2" ); }

extern uint32_t _xtos_ints_on(uint32_t mask);


//__attribute__((noinline))
static uint32_t get_cycle_count(void) {

  uint32_t ccount;

  asm volatile("rsr %0,ccount":"=a" (ccount));
  return ccount;
}

void panic(uint32_t p) {

  int i;

  chSysLock();
  while (true) {
    for (i=0; i<p; i++) {
      PIN_OUT_SET = _BV(2);           /* GPIO2 led on */
      call_delay_us( 50000 );
      PIN_OUT_CLEAR = _BV(2);         /* GPIO2 led on */
      call_delay_us( 500000 );
    }
    call_delay_us( 2000000 );
  }
}

static THD_WORKING_AREA(waThread1, 256);
static THD_FUNCTION(Thread1, arg) {

  (void)arg;
  chRegSetThreadName("blinker");
  while (true) {
    PIN_OUT_SET = _BV(2);         /* GPIO2 led on */
    chThdSleepMilliseconds(50);
    PIN_OUT_CLEAR = _BV(2);       /* GPIO2 led off */
    chThdSleepMilliseconds(950);
  }
}

static THD_WORKING_AREA(waThread2, 1024);
static THD_FUNCTION(Thread2, arg) {

  (void)arg;
  chRegSetThreadName("uart");
  uint32_t i = 0;
  volatile uint32_t cycles;

  while (true) {
    cycles = get_cycle_count();
//    printf("i = %ld, cycles = %lu\n", i++, get_cycle_count());
    printf("i = %ld\n", i++);
//    printf("%08lX\n", cycles);
//    call_delay_us( 15000 );

/*
 doesn't work with more than 12 chars, exception: unaligned load or store
 at port_switch -> l32i.n   a1,  a2, 12
*/
//    printf("1234567890123");
/*
    printf("%08lX ", *(uint32_t *)0x3FFE9000);
    chThdSleepMilliseconds(10);
    printf("%08lX\n", *(uint32_t *)0x3FFE9004);
*/
    chThdSleepMilliseconds(500);
  }
}

void set_vectors(uint32_t vptr) {

  asm volatile( "wsr.vecbase  %0\n" : /*out*/ : /*in*/ "r" (vptr) : /*clobber*/ );
}

int main() {

	nosdk8266_init();

  /* set vectors base */
  set_vectors(0x40100000);

  /* init gpio2 */
	PIN_FUNC_SELECT(PERIPHS_IO_MUX_GPIO2_U,FUNC_GPIO2);
	PIN_DIR_OUTPUT = _BV(2);        /* GPIO2 as output */
  PIN_OUT_CLEAR = _BV(2);         /* GPIO2 led off */

  /* set intlevel to 15 */
  asm volatile( "rsil a2, 15" );  /* disable all interrupts */

  /* set timer to 1000Hz */
  asm volatile(
    "rsr  a2, ccount        \n"
    "movi a3, 80000         \n"
    "add  a2, a2, a3        \n"
    "wsr  a2, ccompare0     \n"
  );

  /* enable timer interrupt */
  _xtos_ints_on(_BV(6));    /* from core-isa.h XCHAL_TIMER0_INTERRUPT = 6 */

  chSysInit();

  PIN_OUT_SET = _BV(2);           /* GPIO2 led off */
  chThdSleepMilliseconds(1000);
  PIN_OUT_CLEAR = _BV(2);         /* GPIO2 led off */
  chThdSleepMilliseconds(1000);

  chThdCreateStatic(waThread1, sizeof(waThread1), NORMALPRIO+1, Thread1, NULL);
  chThdCreateStatic(waThread2, sizeof(waThread2), NORMALPRIO+2, Thread2, NULL);

  while (1) {
    chThdSleepMilliseconds(1000);
  }
  return 0;
}

